/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2016                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  Copyright (c) 2009-2015                                                **
**  German Research School for Simulation Sciences GmbH,                   **
**  Laboratory for Parallel Programming                                    **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


import scalasca.cubex.cube.*;
import scalasca.cubex.cube.errors.*;
import java.util.*;
import java.lang.*;
import java.lang.String;
import java.io.*;
import scalasca.cubex.cube.services.transformation.*;
import scalasca.cubex.cube.datalayout.data.value.*;

/*
 * Test class for BTarInputStream
 *
 */
public class jcuber_example
{
    public static void main(String[] args)
    {
	boolean is_dump = false;
	if (args.length == 0)
	{
	    usage();
	    return;
	}
	if (args.length >= 2)
	{
	    is_dump = true;
	    if (!args[1].trim().equalsIgnoreCase("dump"))
	    {
		System.out.println("Unknown option: " + args[1]);
		usage();
		return;
	    }
	}
	Cube cube = new Cube();
	try
	{
	    cube.openCubeReport(args[0]);
	    ArrayList<Metric> metrics = cube.get_metv();
	    ArrayList<Metric> root_metrics = cube.get_root_metv();
	    ArrayList<Region> regions = cube.get_regionv();
	    ArrayList<Cnode> cnodes = cube.get_cnodev();
	    ArrayList<Cnode> root_cnodes = cube.get_root_cnodev();
	    ArrayList<SystemTreeNode> machines = cube.get_root_stnv();
        ArrayList<SystemTreeNode> stns = cube.get_stnv();
	    ArrayList<Node> nodes = cube.get_nodev();
	    ArrayList<scalasca.cubex.cube.LocationGroup> lgs = cube.get_location_groupv();
        ArrayList<scalasca.cubex.cube.Location> locs = cube.get_locationv();
	    ArrayList<scalasca.cubex.cube.Cartesian> topologies = cube.get_cartv();

	    System.out.println(
	            "Version:" + cube.get_version() + "\n" + 
			    "Metrics:" + metrics.size() + "\n" +
			    "Root Metrics:" + root_metrics.size() + "\n" +
			    "Regions:" + regions.size() + "\n" +
			    "Cnodes:" + cnodes.size() + "\n" +
			    "SystemTreeNodes:" + stns.size() + "\n" +
			    "Machines:" + machines.size() + "\n" +
			    "LocationGroups:" + lgs.size() + "\n" +
			    "Locationa:" + locs.size() + "\n" +
			    "Topologies:" + topologies.size() 
				);
	    System.out.println("------- Metrics  in " + args[0] + " ------------------");
	    for (Metric met : root_metrics)
	    {
		printMetrics(met, 0);
	    }
        System.out.println("");       
	    System.out.println("------- Regions in " + args[0] + " ------------------");
	    for (Region reg : regions)
	    {
		printRegion(reg);
	    }
        System.out.println("");       
	    System.out.println("------- Calltree  in " + args[0] + " -----------------");
	    for (Cnode cnode : root_cnodes)
	    {
		printCnodes(cnode, 0);
	    }
        System.out.println("");       
	    System.out.println("------- System tree  in " + args[0] + " --------------");
	    for (SystemTreeNode machine : machines)
	    {
		printSTN(machine, 1);
	    }
        System.out.println("");       
	    System.out.println("------- Topologies  in " + args[0] + " --------------");
	    for (scalasca.cubex.cube.Cartesian topology : topologies)
	    {
		printTopology(topology);
	    }

	    if (is_dump)
	    {
		for (scalasca.cubex.cube.Metric met: metrics)
		{
		    System.out.println(" ================================ Metric "+met.getDisplayName() + " ============================");
		    for (scalasca.cubex.cube.Cnode cnode: cnodes)
		    {
			System.out.println(" -------------------------------- Cnode "+cnode.getName() + " ----------------------------");
			for (scalasca.cubex.cube.Location location: locs)
			{
			    System.out.print( cube.get_sev_adv(met, cnode, location).toString() + " ");
			}System.out.println();
		    }
		}
	    }
	}catch (BadSyntaxException e)
	{
		System.out.println("Bad syntax error:" + e.getMessage());
	}catch (BadCubeReportLayoutException e)
	{
		System.out.println("Bad layout error:" + e.getMessage());
	}catch (IOException e)
	{
		System.out.println("IO Error: " + e.getMessage());
	}catch (Exception e)
	{
		System.out.println("General error:" + e.getMessage());
	}
    }




    static public void printMetrics(Metric met, int ident)
    {
        printIdentSpaces(ident);
        System.out.println(met.getUniqName());
        printIdentSpaces(ident);
        Map<String, String> attrs = met.get_attrs();
        for (String key : attrs.keySet()) 
        {
            printIdentSpaces(ident);    
            System.out.println(key+ "=>"+ attrs.get(key));
        }        
        for (Object _met: met.getAllChildren().toArray())
        {
            printMetrics((Metric)_met, ident + 1);
        } 
    }	


    static public void printRegion(Region reg)
    {
        printIdentSpaces(1);
        System.out.println(reg.getName() + ":" + reg.getMangledName() + " (paradigm="+reg.getParadigm() +", role="+reg.getRole() + ")" );
        Map<String, String> attrs = reg.get_attrs();
        for (String key : attrs.keySet()) 
        {
            printIdentSpaces(1);    
            System.out.println(key+ "=>"+ attrs.get(key));
        }
    }



    static public void printCnodes(Cnode cnode, int ident)
    {
	printIdentSpaces(ident);
	System.out.println(cnode.getRegion().getName());
    printIdentSpaces(ident);
    Map<String, String> attrs = cnode.get_attrs();
    for (String key : attrs.keySet()) 
    {
        printIdentSpaces(ident);    
        System.out.println(key+ "=>"+ attrs.get(key));
    }
	for (Object _cnode: cnode.getAllChildren().toArray())
	{
	    printCnodes((Cnode)_cnode, ident + 1);
	}
    }


    static public void printSTN(SystemTreeNode machine, int ident)
    {
	printIdentSpaces(ident);
	System.out.println(machine.getName());
    printIdentSpaces(ident);
	System.out.println(machine.getSystemTreeNodeClass());
    printIdentSpaces(ident);
    Map<String, String> attrs = machine.get_attrs();
    for (String key : attrs.keySet()) 
    {
        printIdentSpaces(ident);    
        System.out.println(key+ "=>"+ attrs.get(key));
    }
    System.out.println("Print all stns");
	for (Object _node: machine.getAllChildren().toArray())
	{
	
	    printSTN((SystemTreeNode)_node, ident+1);
	}
    for (Object _lg: machine.getAllGroups().toArray())
    {
        printLocationGroup((LocationGroup)_lg, ident+1);
    }	
    }


    static public void printLocationGroup(scalasca.cubex.cube.LocationGroup process, int ident)
    {
	printIdentSpaces(ident);
	System.out.println(process.getName());	
    Map<String, String> attrs = process.get_attrs();
    for (String key : attrs.keySet()) 
    {
        printIdentSpaces(ident);    
        System.out.println(key+ "=>"+ attrs.get(key));
        
    }
	for (Object _thread: process.getAllChildren().toArray())
	{
	    printLocation((scalasca.cubex.cube.Location)_thread, ident+1);
	    
	}
    }

    static public void printLocation(scalasca.cubex.cube.Location location, int ident)
    {
        printIdentSpaces(ident);
        System.out.println(location.getName());
        Map<String, String> attrs = location.get_attrs();
        for (String key : attrs.keySet()) 
        {
            printIdentSpaces(ident);    
            System.out.println(key+ "=>"+ attrs.get(key));
            
        }
    }

    static public void printTopology(scalasca.cubex.cube.Cartesian topology)
    {
	System.out.println("Name: " + topology.get_name());
	System.out.println("Number of dimensions: " + topology.get_ndim());
	for (int i=0; i<topology.get_ndim(); i++)
	{
	    System.out.println(i + "th dimension name                  : " + topology.get_dim_name(i));
	    System.out.println(i + "th dimension is periodical         : " + topology.get_periodv().get(i));
	    System.out.println(i + "th dimension number of coordinates : " + topology.get_dimv().get(i));	
	    System.out.println("");
	}

    }


    static private void printIdentSpaces(int num)
    {
	for(int i=0; i< num; i++)
	    System.out.print("\t");
    }





    static void usage()
    {
	System.out.println(
			    "Usage: \n"+
			    "\tjava -cp $CLASSPATH:[path to CubeReader.jar] jcuber_example <cube> [dump]\n "+
				    "\tDumps the structure of the <cube> report into STDOUT. \n "+
			    "Options:\n "+
				"\t cube\t- name of cube file; \n "+
				"\t dump\t- prints the content of the report file. \n"    
			    );
    }



}
